#include <qcursor.h>
#include <qpoint.h>
#include <qstring.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qevent.h>
#include <klocale.h>
#include <kaboutapplication.h>
#include <kiconloader.h>
#include <stdio.h>
#include <stdlib.h>

#include "qtrans.h"
#include "settings.h"
#include "utils.h"
#include "libbab.h"

QTrans::QTrans(QWidget *parent, const char *name) : QWidget(parent, name)
{
  QPixmap pix;

  this->resize(306,196);
  this->setMinimumSize(200,100);

  QString KdeDir = getenv("KDEDIR");
  QString IconPath = KdeDir + "/share/icons/hicolor/22x22/actions/";
  this->setIcon(QPixmap(KdeDir+"/share/icons/mini/qtrans.xpm"));

  bExit= new QPushButton(this,"NoName");
  bExit->setMinimumSize(0,0);
  printf("%s\n",i18n("&Exit"));
  printf("%s\n",i18n("&Exit"));
  if (pix.load(IconPath + "stop.png"))
    bExit->setPixmap(pix);
  else
    bExit->setText(i18n("&Exit"));
  QToolTip::add(bExit,I18N_NOOP(("Exit")));

  bTranslate= new QPushButton(this,"NoName");
  bTranslate->setMinimumSize(0,0);
  if (pix.load(IconPath + "reload.png")) bTranslate->setPixmap(pix);
  else bTranslate->setText(i18n("&Translate"));
  QToolTip::add(bTranslate,i18n("Translate"));

  bAbout= new QPushButton(this,"NoName");
  bAbout->setMinimumSize(0,0);
  if (pix.load(IconPath + "help.png")) bAbout->setPixmap(pix);
  else bAbout->setText(i18n("&About"));
  QToolTip::add(bAbout,i18n("About"));

  bConf = new QPushButton(this,"NoName");
  if (pix.load(IconPath + "configure.png")) bConf->setPixmap(pix);
  else bConf->setText(i18n("&Conf"));
  QToolTip::add(bConf,i18n("Settings"));

  Out= new QTextBrowser(this,"NoName");
  Out->setMinimumSize(0,0);
  //Out->setTextFormat(RichText);

  In= new QLineEdit(this,"NoName");
  In->setMinimumSize(0,0);
  In->setText("");

  connect(bExit,SIGNAL(clicked()),this,SLOT(slot_Quit()));
  connect(bTranslate,SIGNAL(clicked()),this,SLOT(slot_Ok()));
  connect(In,SIGNAL(returnPressed()),this,SLOT(slot_Ok()));
  connect(bAbout,SIGNAL(clicked()),this,SLOT(slot_About()));
  connect(bConf,SIGNAL(clicked()),this,SLOT(slot_Config()));

  clip = kapp->clipboard();
  connect(clip,SIGNAL(dataChanged()),this,SLOT(slot_ClipChanged()));
  Clear_Out();

  Dock = new SystemTray(this,"Dock");
  Dock->setPixmap(QPixmap(IconPath + "reload.png"));
  QToolTip::add(Dock,i18n("QTrans"));


  /*ShortCut = new KGlobalAccel();
  ShortCut->insertItem( i18n("Prova"), "Prova", "CTRL+T" );
  ShortCut->connectItem( "Prova",this, SLOT(slot_ShortCut()));
  ShortCut->readSettings();*/

  /*setMouseTracking(TRUE);
  Out->setMouseTracking(TRUE);
  QApplication::setGlobalMouseTracking( TRUE );*/
 // Mouse = new QMouseEvent(QMouseEvent::MouseButtonRelease, constQPoint&pos, 0,0)
}

QTrans::~QTrans()
{
}

/*-- Translate a QString and Update the View --*/
void QTrans::Translate(QString EngWord){
   Clear_Out();
   babylon Dic;
   if (!Dic.open(English_Path,Lang_Path)) {
     QMessageBox::warning( this, "QTrans",
                      i18n("Could not open the dics!"), i18n("Quit"), 0,0, 1 );
   }
   babylon::container_type lst ;

   Dic.translate(string(EngWord),lst);

   string prev="", prev_def="";
   for ( babylon::container_type::const_iterator
          it = lst.begin() ;
          it != lst.end() ;
          ++ it ) {

   string  attribute;
   assert( it->attrib.size() >= 10 ) ;
   if (it->attrib[3] == '8' && it->attrib[8] == '8') {
       /* intercalare */
       attribute = (i18n("inter."));
       } else if (it->attrib[3] == '2' && it->attrib[8] == '8') {
       /* preposizione */
       attribute = (i18n("prep."));  // Preposition
       } else if ( it->attrib[ 2 ] > '1' || it->attrib[ 3 ] != '0' || it->attrib[4] == '8'
                    || it->attrib[4]=='A' || it->attrib[5]=='2') {
       /* sostantivo (s.) */
       attribute = (i18n("n.")) ;   // Name
       } else if ( it->attrib[ 6 ] != '0' || it->attrib[ 9 ] != '0' ) {
       /* verbo */
       attribute = (i18n("v.")) ;   //  Verb
       } else if ( it->attrib[ 7 ] != '0' ) {
       /* aggettivo (agg.) */
       attribute = (i18n("adj.")) ;  // Adjective
       } else if ( it->attrib[ 8 ] == '1' ) {
       /* avverbio (avv) */
       attribute = (i18n("adv.")) ;   // Adverb
       } else if (it->attrib[4] == '4' || it->attrib[4] == '2' || it->attrib[8]=='8') {
       /* niente!! */
       attribute = ("");
       } else
       attribute = it->attrib ; /******** Must be "" !!!
                                otherwise it display a number if cannot find
                                the attribute
                                however, for now i use it for debug */
   if (attribute!="") attribute = "("+attribute+") ";

   if (it->definition != "") {
      if (AutoPopUp) {
          /*QPoint Pos = QCursor::pos();
          move(Pos.x()+20,Pos.y()+20);*/  // It works!!! ..but must be improved..
          showNormal();setActiveWindow();
          raise();
      }
      else if (!isVisible()) return;
      In->setText(EngWord);
      if (it == lst.begin()) {
         Out->setText("<qt bgcolor=#F7FCAB text=black><b><u><font face=lucida>"+str(it->word)+" :</b>");
         prev=it->word;
      } else if (prev!=it->word) {
           Out->append("<br/><b><u><font face=lucida>"+str(it->word)+" :</b>");
           prev=it->word;
      }
      if (prev_def!=it->definition) {
        Out->append("<i><font face=helvetica color=blue>"+str(attribute)+"</i>"+str_2(it->definition));
        prev_def=it->definition;
      }
   }
   }
   Dic.close();

}

void QTrans::Clear_Out(){
   Out->setText("<qt bgcolor=#F7FCAB text=black></qt>");
}

void QTrans::updateMask(){
   Out->setGeometry(0,35,this->width(),this->height()-35);
   In->setGeometry(5,5,this->width()-180,25);
   bExit->setGeometry(this->width()-35,4,35,28);
   bTranslate->setGeometry(this->width()-170,4,60,28);
   bAbout->setGeometry(this->width()-110,4,40,28);
   bConf->setGeometry(this->width()-70,4,35,28);
}

void QTrans::slot_Quit(){
  close();
}

/*-- Monitor the Clipboard and try to translate its content --*/
void QTrans::slot_ClipChanged() {
   if (AutoPopUp || !isMinimized())
   if (clip->text()!=NULL){
     QString A = tr_str(clip->text());
     if (A!=In->text()) {
        Translate(A);
     }
   }
}

void QTrans::slot_Config() {
   Settings *Set=new Settings(this,"NoName",TRUE);
   Set->show();
}

void QTrans::slot_About() {
   KAboutApplication *aDial = new KAboutApplication(this,"aDial");
   aDial->show();
   //Out->setSource("/usr/share/doc/HTML/index.html");
}

/*-- If the window isn't visble, then show it;
   otherwise hide the window --*/
void QTrans::slot_ShortCut() {
   if (!isVisible() || !isActiveWindow()) {
      showNormal();
      setActiveWindow();
      raise();
   } else {
     showMinimized();
   }
}

void QTrans::slot_Ok() {
   QString s= In->text();
   Translate(s);
}

/*-- Intercept the Close Event and decide
  if had to close or minimize --*/
void QTrans::closeEvent(QCloseEvent *event) {
   if (!isVisible() || (!DontClose)) KApplication::exit(0);
   else showMinimized();
}
